@@ -5,7 +5,7 @@ use warnings;
 use Carp qw/croak/;
 use XSLoader;
 
-our $VERSION = '3.014'; # Don't forget to update the TestCompat set for testing against installed encoders!
+our $VERSION = '3.015'; # Don't forget to update the TestCompat set for testing against installed encoders!
 our $XS_VERSION = $VERSION; $VERSION= eval $VERSION;
 
 # not for public consumption, just for testing.
@@ -114,7 +114,7 @@ sub decode_from_file {
     my ($self, $file, )= @_; # pos 3 is "target var" if one is provided
     open my $fh, "<", $file
         or die "Failed to open '$file' for read: $!";
-    my $buf= do{ local $/; <> };
+    my $buf= do{ local $/; <$fh> };
     close $fh
         or die "Failed to close '$file': $!";
     if (wantarray && ($self->flags & SRL_F_DECODER_DESTRUCTIVE_INCREMENTAL)) {
